// © 2023, CGVR (https://cgvr.informatik.uni-bremen.de/),
// Author: Andre Mühlenbrock (muehlenb@uni-bremen.de)

#pragma once

#include "src/math/Vec4f.h"
#include "src/scene/Surface.h"

/** Defines a single ball which can be part of a Metaball surface */
struct Ball
{
    /** Weight of the ball */
    float weight = 1.0;

    /** Center position of the ball */
    Vec4f center;

    /** Bandwidth of the ball */
    float bandwidth = 1.0;

    /** Constructs a ball with the given parameters */
    Ball(float pWeight, const Vec4f pCenter, float pBandwidth){
        weight = pWeight;
        center = pCenter;
        bandwidth = pBandwidth;
    }
};

/**
 * A Metaball surface.
 */
class Metaball : public Surface {
    /** Isovalue*/
    float isovalue = 1.0;

    /** Contains all balls that form the surface of the metaball */
    std::vector<Ball> balls;

    /**
     * Computes the weighted sum of the potential functions of all balls
     *
     * @param x  This is the point for which to evaluate the potential value.
     * @return Value of the potential function.
     */
    float F(Vec4f vec) const;

public:
    /** Metaball */
    Metaball(std::vector<Ball> balls, float isovalue, Shader shader);

    /** See parent class 'Surface.h' for documentation */
    virtual Intersection intersect(Ray ray, float tStart, float tEnd) override;

    /** Adds a ball to this metaball */
    void addBall(const Ball &ball) { balls.push_back(ball); }
};
